import { Injectable } from '@angular/core';
import Localbase from 'localbase';
import { Character } from 'src/interfaces/character';
import { Router } from '@angular/router';

@Injectable({
  providedIn: 'root',
})
export class DataService {
  db: any;

  constructor(private Router: Router) {
    this.db = new Localbase('myDatabase');
    // TODO: Fetch all data from database and write it to a character object
  }

  public character: any;

  public addData(collection: string, data: any, key?: string): void {
    if (key) {
      this.db.collection(collection).add(data, key);
    } else {
      this.db.collection(collection).add(data);
    }
  }

  public getData(collection: string, key: string) {
    return this.db.collection(collection).doc(key).get();
  }

  public getCollection(collection: string) {
    return this.db.collection(collection).get();
  }

  public getCollectionWithKeys(collection: string) {
    return this.db.collection(collection).get({ keys: true });
  }

  public deleteCollection(collection: string) {
    this.db.collection(collection).delete();
  }

  public async selectCharacter(character: string) {
    const characterData = await this.getCollectionWithKeys(character);
    this.character = this.getcurrentCharacterData(characterData);
    this.Router.navigate(['journal']);
  }

  private getcurrentCharacterData(currentCharacterData: any) {
    // let [appearanceData, attributesData, characterData, combatStatsData, personalityData, skillsData, weaponsData];

    const [
      appearanceData,
      attributesData,
      characterData,
      combatStatsData,
      personalityData,
      skillsData,
      weaponsData,
    ] = currentCharacterData.map((entry: any) => entry.data);

    const appearance = {
      age: appearanceData.age,
      height: appearanceData.height,
      weight: appearanceData.weight,
      eyes: appearanceData.eyes,
      skin: appearanceData.skin,
      hair: appearanceData.hair,
    };

    const attributes = {
      strength: attributesData.strength,
      dexterity: attributesData.dexterity,
      constitution: attributesData.constitution,
      intelligence: attributesData.intelligence,
      wisdom: attributesData.wisdom,
      charisma: attributesData.charisma,
    };

    const character = {
      name: characterData.name,
      race: characterData.race,
      class: characterData.class,
      subclass: characterData.subclass,
      level: characterData.level,
      background: characterData.background,
      experience: characterData.experience,
      inspiration: characterData.inspiration,
      proficiencyBonus: characterData.proficiencyBonus,
    };

    const combatStats = {
      armorClass: combatStatsData.armorClass,
      initiative: combatStatsData.initiative,
      speed: combatStatsData.speed,
      hitPointMaximum: combatStatsData.hitPointMaximum,
      currentHitPoints: combatStatsData.currentHitPoints,
      temporaryHitPoints: combatStatsData.temporaryHitPoints,
      hitDice: combatStatsData.hitDice,
      deathSaveSuccesses: combatStatsData.deathSaveSuccesses,
      deathSaveFailures: combatStatsData.deathSaveFailures,
    };

    const personality = {
      personalityTraits: personalityData.personalityTraits,
      ideals: personalityData.ideals,
      bonds: personalityData.bonds,
      flaws: personalityData.flaws,
    };

    const skills = {
      acrobatics: skillsData.acrobatics,
      animalHandling: skillsData.animalHandling,
      arcana: skillsData.arcana,
      athletics: skillsData.athletics,
      deception: skillsData.deception,
      history: skillsData.history,
      insight: skillsData.insight,
      intimidation: skillsData.intimidation,
      investigation: skillsData.investigation,
      medicine: skillsData.medicine,
      nature: skillsData.nature,
      perception: skillsData.perception,
      performance: skillsData.performance,
      persuasion: skillsData.persuasion,
      religion: skillsData.religion,
      sleightOfHand: skillsData.sleightOfHand,
      stealth: skillsData.stealth,
      survival: skillsData.survival,
    };
    const weapons = {
      name: weaponsData.name,
      attackBonus: weaponsData.attackBonus,
      damage: weaponsData.damage,
      damageType: weaponsData.damageType,
      range: weaponsData.range,
      description: weaponsData.damageDice,
    };
    const newCharacter: Character = {
      appearance: appearance,
      attributes: attributes,
      characterData: character,
      combatStats: combatStats,
      personality: personality,
      skills: skills,
      weapons: weapons,
    };
    this.character = newCharacter;
  }
}
